@@ -23,6 +23,7 @@ from .exceptions import (
     FritzResourceError,
     FritzServiceError,
 )
+from .fritzhttp import FritzHttp
 from .soaper import Soaper
 from .utils import (
     get_bool_env,
@@ -88,6 +89,37 @@ class FritzConnection:
     FRITZ_USERNAME and FRITZ_PASSWORD settings and taken from there, if
     found.
 
+    Basic usage assuming `user` and `password` stored in the environment:
+
+    >>> fc = FritzConnection(address="192.168.178.1")
+    >>> fc.call_action("WANIPConn1", "ForceTermination", arguments={})
+
+    This will reconnect the router with the external network.
+    `arguments` is not necessary here, but in case where arguments must
+    be provided, this is done by `arguments` (see also the documentation
+    for the `call_action()`` method`). The `call_action()` method is
+    used for the TR-064 API and returns a dictionary with the results.
+
+    For accessing the http-interface (aka AHA-HTTP-Interface) of the
+    router, FritzConnection provides the `call_http()` method (added in
+    version 1.12). As arguments this method takes a required command
+    (like `getswitchlist`) and optional parameters as described in the
+    AVM documentation:
+
+    >>> fc.call_http("getswitchlist")
+
+    This method triggers a http response and returns a dictionary with
+    three key-value pairs: the `content-type`, the `encoding` and the
+    `content` itself. The values are all of type string. The
+    content-type is typically "text/plain" or "text/xml", the encoding,
+    typically "utf-8".
+
+    The method will raise a FritzAuthorizationError in case of missing
+    credentials. In case of an unknown command or identifier a
+    FritzHttpInterfaceError will get raised.
+
+    .. versionadded:: 1.12
+
     The optional parameter `timeout` is a floating number in seconds
     limiting the time waiting for a router response. This is a global
     setting for the internal communication with the router. In case of a
@@ -248,6 +280,8 @@ class FritzConnection:
         )
         # set default user for FritzOS >= 7.24:
         self._reset_user(user, password)
+        # provide also the http-interface for more homeautomation tasks
+        self.http_interface = FritzHttp(self)
 
     def __repr__(self):
         """Return a readable representation"""
@@ -391,6 +425,10 @@ class FritzConnection:
         "WLANConfiguration:2" converts to "WLANConfiguration2"". Invalid
         service names will raise a ServiceError and invalid action names
         will raise an ActionError.
+
+        The method returns a dictionary with argument-names as keys and
+        the corresponding information as values. Numeric and boolean
+        values are converted from strings to Python datatypes.
         """
         arguments = arguments if arguments else dict()
         if not arguments:
@@ -402,6 +440,51 @@ class FritzConnection:
             raise FritzServiceError(f'unknown service: "{service_name}"')
         return self.soaper.execute(service, action_name, arguments)
 
+    def call_http(
+        self,
+        command,
+        identifier=None,
+        **kwargs
+    ):
+        """
+        Excecutes the given command by means of the http-interface. This
+        can be useful for homeautomation-task currently not provided by
+        the TR-064 interface. The `identifier` represents the `ain` of a
+        target-device. `kwargs` can hold additional parameters depending
+        on the device.
+
+        The method returns a dictionary of strings with three items: the
+        `content-type`, the `encoding` and the corresponding result (the
+        `content`). The content-type is typically "text/plain" or
+        "text/xml", the encoding, typically "utf-8".
+
+        The method will raise a FritzAuthorizationError in case of
+        missing credentials. In case of an unknown command or identifier
+        a FritzHttpInterfaceError will get raised.
+
+        Note: information and interactions with actors provided by
+        TR-064 and the http-interface partly overlap. If response time
+        is an issue, TR-064 should be preferred because at time of
+        writing a call to `call_action` (the TR-064 interface) is about
+        5 to 6 times faster than calling `call_http` (the
+        http-interface).
+
+        .. versionadded:: 1.12
+        """
+        header, content = self.http_interface.execute(
+            command,
+            identifier,
+            **kwargs
+        )
+        content_type, charset = [item.strip() for item in header.split(";")]
+        # extract the encoding from the charset-information
+        encoding = charset.split("=")[-1].strip()
+        return {
+            "content-type": content_type,
+            "encoding": encoding,
+            "content": content
+        }
+
     def reconnect(self):
         """
         Terminate the connection and reconnects with a new ip.
@@ -502,8 +585,11 @@ class FritzConnection:
             raise FritzConnectionException(message)
         filename = f"{address}{FRITZ_CACHE_EXT}.{suffix}"
         if cache_directory:
-            return Path(cache_directory) / filename
-        return Path().home() / FRITZ_CACHE_DIR /filename
+            path = Path(cache_directory)
+        else:
+            path = Path().home() / FRITZ_CACHE_DIR
+        path.mkdir(exist_ok=True)
+        return path / filename
 
     def _write_api_to_cache(self, path, cache_format):
         """
